/* Copyright (C) 1989, 1995, 1996, 1997, 1998 Aladdin Enterprises.  All rights reserved.

   This file is part of Aladdin Ghostscript.

   Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
   or distributor accepts any responsibility for the consequences of using it,
   or for whether it serves any particular purpose or works at all, unless he
   or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
   License (the "License") for full details.

   Every copy of Aladdin Ghostscript must include a copy of the License,
   normally in a plain ASCII text file named PUBLIC.  The License grants you
   the right to copy, modify and redistribute Aladdin Ghostscript, but only
   under certain conditions described in the License.  Among other things, the
   License requires that the copyright notice and this notice be preserved on
   all copies.
 */

/* #define VERBOSE */

/*$Id: gdevphoton.c $ */
/* QNX-Photon driver for Ghostscript library */


#include "gx.h"			/* for gx_bitmap; includes std.h */
#include "math_.h"
#include "memory_.h"
#include <Ph.h>
#include <Pt.h>
#include "gserrors.h"
#include "gsmatrix.h"		/* for gscoord.h */
#include "gscoord.h"		/* for gs_currentmatrix */
#include "gsdevice.h"		/* for gs_currentdevice */
#include "gsparam.h"
#include "gxdevice.h"
#include "gxpath.h"
#include "gxgetbit.h"
#include "gxiparam.h"
#include "gsiparm2.h"




typedef struct gx_device_photon_s {
	gx_device_common;
	bool		IsPageDevice;
	PtWidget_t*	Window;
} gx_device_photon;

#define phdev ((gx_device_photon *)dev)

/* These are adjusted later */                    		

/* A4: 8.3" x 11.7" */
/* #define XSIZE 595 */
/* #define YSIZE 842 */
#define XPPI  80.0			
#define YPPI  72.0			
#define XSIZE (83*XPPI/10)
#define YSIZE (117*YPPI/10)



/* Forward references */

private int photon_open(gx_device *dev);
private int photon_close(gx_device *dev);
private int photon_sync_output(gx_device *dev);
private int photon_fill_rectangle(gx_device *dev, int x1, int y1, int w, int h, gx_color_index color);
private int photon_copy_mono(gx_device *dev, byte *base, int sourcex, int raster, gx_bitmap_id id, int x, int y, int w, int h, gx_color_index zero, gx_color_index one);
private int photon_copy_color(gx_device *dev,  byte *base, int sourcex, int raster, gx_bitmap_id id,  int x, int y, int w, int h);
private int photon_sync_output(gx_device *dev);
private gx_color_index photon_map_rgb_color(gx_device *dev, gx_color_value r, gx_color_value g,  gx_color_value b);
private int photon_map_color_rgb(gx_device *dev, gx_color_index color, gx_color_value prgb[3]);
private int photon_output_page(gx_device *dev, int num_copies, int flush);
private int photon_get_params(gx_device * dev, gs_param_list * plist);
private int photon_put_params(gx_device * dev, gs_param_list * plist);
private gx_device* photon_get_page_device(gx_device *dev);




/* The device descriptor */
private gx_device_procs photon_procs =
{
    photon_open,
    gx_default_get_initial_matrix,
    photon_sync_output,
    photon_output_page,
    photon_close,
    photon_map_rgb_color,
    photon_map_color_rgb,
    photon_fill_rectangle,
    gx_default_tile_rectangle,			/* tile_rectangle */
    photon_copy_mono,
    photon_copy_color,
	NULL,							/* draw line */
	NULL,							/* draw bits */
	photon_get_params,
	photon_put_params,
    NULL,			/* map_cmyk_color */
    NULL,			/* get_xfont_procs */
    NULL,			/* get_xfont_device */
    NULL,			/* map_rgb_alpha_color */
    photon_get_page_device,			/* get_page_device */
    NULL,			/* get_alpha_bits */
    NULL,			/* copy_alpha */
    NULL,			/* get_band */
    NULL,			/* copy_rop */
    NULL,			/* fill_path */
    NULL,			/* stroke_path */
    NULL,			/* fill_mask */
    NULL,			/* fill_trapezoid */
    NULL,			/* fill_parallelogram */
    NULL,			/* fill_triangle */
    NULL,			/* draw_thin_line */
    NULL,			/* begin_image */
    NULL,			/* image_data */
    NULL,			/* end_image */
    NULL,			/* strip_tile_rectangle */
    NULL,			/* strip_copy_rop */
    NULL,			/* get_clipping_box */
    NULL,			/* begin_typed_image */
    NULL,			/* get_bits_rectangle */
};

const gx_device_photon gs_photon_device = 
{	
  std_device_color_body(gx_device_photon, &photon_procs, "photon",
  		XSIZE,YSIZE,	/* x and y extent (nominal) */
		XPPI,YPPI,		/* x and y density (nominal) */
		/*dci_color( */ 24, 255, 256 /*) */ ),
	{0},	/* std_procs */
	1,		/* IsPageDevice */
	NULL,	/* -> Window-Widget */
};

PhGC_t		*gc, *old_gc;
static int	isOpen = 0;

private int photon_open(gx_device *dev)
{		 
	PhPoint_t	point;
	PtArg_t		arg[3];
	PtWidget_t	*window;
    PhRegion_t  info;

	#ifdef VERBOSE
	  fprintf(stderr,"photon_open():\n");
	#endif

	if(isOpen == 1)
		{
		#ifdef VERBOSE
		  fprintf(stderr,"\talready open!\n");
		#endif
		return(0);
		}
			
	if( PtInit( NULL)) {
		fprintf(stderr,"\n\nPhoton is not running, Please select another device.\n");
		return(-1);
		}  
	PhEventArm();
	point.x = dev->width;
	point.y = dev->height;
	
	PtSetArg(&arg[0],Pt_ARG_DIM,&point,0);
	PtSetArg(&arg[1],Pt_ARG_WINDOW_FORCE_FRONT,Pt_TRUE,0);
	window = PtCreateWidget(PtWindow,NULL,2,arg); 
	PtRealizeWidget(window);
	
	phdev->Window = window; 

    memset(&info,0,sizeof(info));			
    info.events_sense = 0;
    info.events_opaque = 0xffffFFFF;
    info.rid = window->rid;
    PhRegionChange(Ph_REGION_EV_SENSE,0,&info,NULL,NULL);
	gc     = PgCreateGC( 0 );
	old_gc = PgSetGC( gc );
	
	PgSetRegion( window->rid );
	/***********************************************/
	/*   Some  PS  files open the device twice so  */
	/*   to  preven  this  we  flag the fact that  */
	/*   we are open.                              */
	/***********************************************/
	isOpen = 1;		
	
	#ifdef VERBOSE
	  fprintf(stderr,"\tOK!\n");
	#endif
	return 0; /*success */	

}

private int photon_close(gx_device *dev)
{  	
	#ifdef VERBOSE
	  fprintf(stderr,"photon_close():\n");
	#endif
	/* restore GC */
	PgSetGC( old_gc );
	
	return 0;
}

private gx_device* photon_get_page_device(gx_device *dev)
{  	
	#ifdef VERBOSE
	  fprintf(stderr,"photon_get_page_device():\n");
	#endif
	
	/* This is *always* a page device! KHO, 30-Jun-1999 */
	
	return dev;
}


/* Get the device parameters.  See below. */
private int photon_get_params(gx_device * dev, gs_param_list * plist)
{
    int code = gx_default_get_params(dev, plist);
    long id = (long)(phdev->Window);

	#ifdef VERBOSE
	  fprintf(stderr,"photon_get_params():\n");
	#endif

    if (code < 0 ||
	(code = param_write_long(plist, "WindowID", &id)) < 0 ||
	(code = param_write_bool(plist, ".IsPageDevice", &(phdev->IsPageDevice))) < 0
	)
	DO_NOTHING;

    return code;
}


/* Set the device parameters.  We reimplement this so we can resize */
/* the window and avoid closing and reopening the device, and to add */
/* .IsPageDevice. */
private int photon_put_params(gx_device * dev, gs_param_list * plist)
{
	long id;
	bool is_page;
	int code,ecode;

	#ifdef VERBOSE
	  fprintf(stderr,"photon_put_params():\n");
	#endif

    /* Handle extra parameters */
    switch (code = param_read_long(plist, "WindowID", &id)) {
	case 0:
	case 1:
	    break;
	default:
	    ecode = code;
	    param_signal_error(plist, "GSVIEW", ecode);
    }

    switch (code = param_read_bool(plist, ".IsPageDevice", &is_page)) {
	case 0:
	case 1:
	    break;
	default:
	    ecode = code;
	    param_signal_error(plist, "GSVIEW", ecode);
    }

    if (ecode < 0)
	return ecode;

    phdev->IsPageDevice = is_page;

    code=gx_default_put_params(dev,plist);
	
    return code;
}



private int photon_fill_rectangle(gx_device *dev, int x1, int y1, int w, int h, gx_color_index color)
{
	fit_fill(dev, x1, y1, w, h);
	
	PgSetFillColor(color | Pg_MIX_COLOR);
	PgSetStrokeColor(color | Pg_MIX_COLOR);
	PgDrawIRect(x1,y1,x1+w,y1+h,Pg_DRAW_FILL_STROKE);

	return 0;
}

private int photon_copy_mono(gx_device *dev, byte *base, int sourcex, int raster, gx_bitmap_id id, int x, int y, int w, int h, gx_color_index zero, gx_color_index one)
{				
	PhPoint_t	pos,area;
	int i; 
	int nbytes = (h * raster);

	#ifdef VERBOSE
	  fprintf(stderr,"photon_copy_mono():\n");
	#endif

	fit_copy(dev, base, sourcex, raster, id, x, y, w, h);

	pos.x = x;
	pos.y = y;
	
	area.x = w;
	area.y = h;
	
	if (one != gx_no_color_index) 
	{	
		PgSetTextColor(one);
		PgDrawBitmap(base,0,&pos,&area,raster,0);
	}

	if (zero != gx_no_color_index)
	{
		for (i = 0; i < nbytes; i++) base[i] = ~base[i];
		PgSetTextColor(zero);
		PgDrawBitmap(base,0,&pos,&area,raster,0);
	}

	return 0;

}				
/***************************INCOMPLETE !!!!****************************/
/*   Copy  a  color  pixelmap.  This  is  just like a bitmap, except  */
/*   that  each  pixel  takes 8 bits instead of 1.                    */
/*   I have yet to find a situation that calls this (wmb)             */
/**********************************************************************/
private int photon_copy_color(gx_device *dev,  byte *base, int sourcex, int raster, gx_bitmap_id id,  int x, int y, int w, int h)
{	
	PhPoint_t	pos,area;
	int i; 

	#ifdef VERBOSE
	  fprintf(stderr,"photon_copy_color():\n");
	#endif

	fit_copy(dev, base, sourcex, raster, id, x, y, w, h);

	pos.x = x;
	pos.y = y;
	
	area.x = w;
	area.y = h;
	
	PgDrawBitmap(base,0,&pos,&area,raster,0);
	
	return 0;
}

private int photon_sync_output(gx_device *dev)
{  
	#ifdef VERBOSE
	  fprintf(stderr,"photon_sync_output():\n");
	#endif

	PgFlush();
	return 0;
}

private gx_color_index photon_map_rgb_color(gx_device *dev, gx_color_value r, gx_color_value g,  gx_color_value b)
{    
	#ifdef VERBOSE
	  fprintf(stderr,"photon_map_rgb_color():\n");
	#endif

   return (PgRGB(r/256,g/256,b/256));
	
}    

/* I have yet to find a situation that calls this (wmb)*/
private int photon_map_color_rgb(gx_device *dev, gx_color_index color, gx_color_value prgb[3])
{		
	#ifdef VERBOSE
	  fprintf(stderr,"photon_map_color_rgb():\n");
	#endif

	return 0;
}

private int photon_output_page(gx_device *dev, int num_copies, int flush)
{
	#ifdef VERBOSE
	  fprintf(stderr,"photon_output_page():\n");
	#endif
	PgFlush();
	return 0;
}

